@@ -24,10 +24,11 @@
 
 /* Must include these two first */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
+#include <atk/atkobject.h>
+#include <atk/atkimage.h>
+#include <atk/atkrelationset.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkvbox.h>
@@ -237,11 +238,13 @@ static void
 gnome_print_copies_selection_init (GnomePrintCopiesSelection *gpc)
 {
 	GtkWidget *table, *label, *frame;
-	GtkLabel *collate_label;
 	GtkAdjustment *adj;
-	guint label_key;
 	guint collate_key;
 	GdkPixbuf *pb;
+	AtkRelationSet *relation_set;
+	AtkRelation *relation;
+	AtkObject *relation_targets[1];
+	AtkObject *atko;
 
 #if 0
 	gpc->accel_group = gtk_accel_group_new ();
@@ -256,9 +259,7 @@ gnome_print_copies_selection_init (GnomePrintCopiesSelection *gpc)
 	gtk_container_add (GTK_CONTAINER(frame), GTK_WIDGET (table));
 	gtk_widget_show (table);
 
-	label = gtk_label_new("");
-	label_key = gtk_label_parse_uline (GTK_LABEL (label),
-					   _("N_umber of copies:"));
+	label = gtk_label_new_with_mnemonic (_("N_umber of copies:"));
 	gtk_widget_show (label);
 	gtk_table_attach_defaults ((GtkTable *)table, label, 0, 1, 0, 1);
 
@@ -266,35 +267,34 @@ gnome_print_copies_selection_init (GnomePrintCopiesSelection *gpc)
 	gpc->copies = gtk_spin_button_new (adj, 1.0, 0);
 	gtk_widget_show (gpc->copies);
 	gtk_table_attach_defaults ((GtkTable *)table, gpc->copies, 1, 2, 0, 1);
-#if 0
-	if (label_key != GDK_VoidSymbol) {
-		gtk_widget_add_accelerator
-			(GTK_WIDGET (&GTK_SPIN_BUTTON (gpc->copies)->entry),
-			 "grab_focus", gpc->accel_group, label_key,
-			 GDK_MOD1_MASK, 0);
-	}
-#endif
+
+	gtk_label_set_mnemonic_widget ((GtkLabel *) label, gpc->copies);
+
+	/* Add a LABELLED_BY relation from the entry to the label. */
+	atko = gtk_widget_get_accessible (gpc->copies);
+	relation_set = atk_object_ref_relation_set (atko);
+	relation_targets[0] = gtk_widget_get_accessible (label);
+	relation = atk_relation_new (relation_targets, 1,
+				     ATK_RELATION_LABELLED_BY);
+	atk_relation_set_add (relation_set, relation);
+	g_object_unref (G_OBJECT (relation));
+	g_object_unref (G_OBJECT (relation_set));
 
 	pb = gdk_pixbuf_new_from_xpm_data (nocollate_xpm);
 	gpc->collate_image = gtk_image_new_from_pixbuf (pb);
 	gdk_pixbuf_unref (pb);
 	gtk_widget_show (gpc->collate_image);
 	gtk_table_attach_defaults ((GtkTable *)table, gpc->collate_image, 0, 1, 1, 2);
+	atko = gtk_widget_get_accessible (gpc->collate_image);
+	atk_image_set_image_description (ATK_IMAGE (atko), _("Image showing the collation sequence when multiple copies of the document are printed"));
 
-	gpc->collate = gtk_check_button_new_with_label("");
-	collate_label = GTK_LABEL (GTK_BIN (gpc->collate)->child);
-	collate_key = gtk_label_parse_uline (collate_label, _("_Collate"));
-#if 0
-	if (collate_key != GDK_VoidSymbol) {
-		gtk_widget_add_accelerator (gpc->collate, "clicked",
-					   gpc->accel_group, collate_key,
-					   GDK_MOD1_MASK, 0);
-	}
-#endif
-
+	gpc->collate = gtk_check_button_new_with_mnemonic (_("_Collate"));
 	gtk_widget_show(gpc->collate);
 	gtk_table_attach_defaults((GtkTable *)table, gpc->collate, 1, 2, 1, 2);
 
+	atko = gtk_widget_get_accessible (gpc->collate);
+	atk_object_set_description (atko, _("If copies of the document are printed separately, one after another, rather than being interleaved"));
+
 	gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
 			    GTK_SIGNAL_FUNC (copies_changed), gpc);
 	gtk_signal_connect (GTK_OBJECT (gpc->collate), "toggled", 
