@@ -28,12 +28,12 @@
 
 #define noGPMP_VERBOSE
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <math.h>
 #include <libart_lgpl/art_affine.h>
+#include <atk/atkobject.h>
+#include <atk/atkrelationset.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkstock.h>
@@ -49,7 +49,6 @@
 #include <libgnomeprint/gnome-print-meta.h>
 
 #include "gnome-print-i18n.h"
-
 #include "gnome-print-master-preview.h"
 #if 0
 #include "gnome-print-preview-icons.h"
@@ -184,7 +183,7 @@ goto_page (GnomePrintMasterPreview *mp, gint page)
 
 	g_snprintf (c, 32, "%d", page + 1);
 	gtk_entry_set_text (GTK_ENTRY (priv->page_entry), c);
-	
+
 	gtk_widget_set_sensitive (mp->bpf, (page != 0));
 	gtk_widget_set_sensitive (mp->bpp, (page != 0));
 	gtk_widget_set_sensitive (mp->bpn, (page != (mp->pages - 1)));
@@ -223,7 +222,7 @@ gpmp_zoom (GnomePrintMasterPreview *mp, gdouble factor, gboolean relative)
 	gdouble zoom;
 
 	priv = (GPMPPrivate *) mp->priv;
-	
+
 	if (relative) {
 		zoom = mp->zoom * factor;
 	} else {
@@ -345,7 +344,7 @@ preview_file_print_cmd (void *unused, GnomePrintMasterPreview *pmp)
 	GPMPPrivate *priv;
 
 	priv = (GPMPPrivate *) pmp->priv;
-	
+
 	gnome_print_master_print (priv->master);
 
 	/* fixme: should we clean ourselves up now? */
@@ -387,7 +386,7 @@ preview_last_page_cmd (void *unused, GnomePrintMasterPreview *pmp)
 	GPMPPrivate *priv;
 
 	priv = (GPMPPrivate *) pmp->priv;
-	
+
 	goto_page (pmp, priv->pagecount - 1);
 }
 
@@ -417,7 +416,7 @@ preview_zoom_fit_cmd (GtkToggleButton *t, GnomePrintMasterPreview *mp)
 
 	zoomx = width / (mp->paw + 5.0 + PAGE_PAD);
 	zoomy = height / (mp->pah + 5.0 + PAGE_PAD);
-	
+
 	gpmp_zoom (mp, MIN (zoomx, zoomy), FALSE);
 }
 
@@ -501,32 +500,32 @@ preview_canvas_key (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	case GDK_KP_Right:
 	case GDK_Right:
 		if (event->state & GDK_SHIFT_MASK)
-			x+=width;
+			x += width;
 		else
-			x+=10;
-		domove=1;
+			x += 10;
+		domove = 1;
 		break;
 	case GDK_KP_Left:
 	case GDK_Left:
 		if (event->state & GDK_SHIFT_MASK)
-			x-=width;
+			x -= width;
 		else
-			x-=10;
-		domove=1;
+			x -= 10;
+		domove = 1;
 		break;
 	case GDK_KP_Up:
 	case GDK_Up:
 		if (event->state & GDK_SHIFT_MASK)
 			goto page_up;
-		y-=10;
-		domove=1;
+		y -= 10;
+		domove = 1;
 		break;
 	case GDK_KP_Down:
 	case GDK_Down:
 		if (event->state & GDK_SHIFT_MASK)
 			goto page_down;
-		y+=10;
-		domove=1;
+		y += 10;
+		domove = 1;
 		break;
 	case GDK_KP_Page_Up:
 	case GDK_Page_Up:
@@ -534,7 +533,7 @@ preview_canvas_key (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	case GDK_KP_Delete:
 	case GDK_BackSpace:
 	page_up:
-		if (y<=0) {
+		if (y <= 0) {
 			if (priv->current_page > 0) {
 				goto_page (pmp, priv->current_page - 1);
 				y = GTK_LAYOUT (priv->canvas)->height - height;
@@ -542,7 +541,7 @@ preview_canvas_key (GtkWidget *widget, GdkEventKey *event, gpointer data)
 		} else {
 			y -= height;
 		}
-		domove=1;
+		domove = 1;
 		break;
 	case GDK_KP_Page_Down:
 	case GDK_Page_Down:
@@ -556,22 +555,22 @@ preview_canvas_key (GtkWidget *widget, GdkEventKey *event, gpointer data)
 		} else {
 			y += height;
 		}
-		domove=1;
+		domove = 1;
 		break;
 	case GDK_KP_Home:
 	case GDK_Home:
 		goto_page (pmp, 0);
-		y=0;
-		domove=1;
+		y = 0;
+		domove = 1;
 		break;
 	case GDK_KP_End:
 	case GDK_End:
-		goto_page (pmp, priv->pagecount-1);
-		y=0;
-		domove=1;
+		goto_page (pmp, priv->pagecount - 1);
+		y = 0;
+		domove = 1;
 		break;
 	case GDK_Escape:
-		g_object_unref (G_OBJECT (pmp));
+		gtk_widget_destroy (GTK_WIDGET (pmp));
 		return TRUE;
 	default:
 		return FALSE;
@@ -588,9 +587,13 @@ static void
 create_preview_canvas (GnomePrintMasterPreview *mp)
 {
 	GPMPPrivate *priv;
-	GtkWidget *status;
+	GtkWidget *status, *l;
 	GnomeCanvasItem *item;
 	GnomePrintConfig *config;
+	AtkRelationSet *relation_set;
+	AtkRelation *relation;
+	AtkObject *relation_targets[1];
+	AtkObject *atko;
 
 	priv = (GPMPPrivate *) mp->priv;
 
@@ -599,6 +602,9 @@ create_preview_canvas (GnomePrintMasterPreview *mp)
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 	priv->canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
 	gtk_widget_pop_colormap ();
+	atko = gtk_widget_get_accessible (GTK_WIDGET (priv->canvas));
+	atk_object_set_name (atko, _("Page Preview"));
+	atk_object_set_description (atko, _("The preview of a page in the document to be printed"));
 
 	gtk_signal_connect (GTK_OBJECT (priv->canvas), "button_press_event",
 			    GTK_SIGNAL_FUNC (preview_canvas_button_press), mp);
@@ -635,17 +641,37 @@ create_preview_canvas (GnomePrintMasterPreview *mp)
 	gnome_canvas_set_scroll_region (priv->canvas, 0 - PAGE_PAD, 0 - PAGE_PAD, mp->paw + PAGE_PAD, mp->pah + PAGE_PAD);
 
 	status = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (status), gtk_label_new (_ ("Page: ")), FALSE, FALSE, 0);
+	l = gtk_label_new_with_mnemonic (_ ("P_age: "));
+	gtk_box_pack_start (GTK_BOX (status), l, FALSE, FALSE, 4);
 	priv->page_entry = gtk_entry_new ();
 	gtk_widget_set_usize (priv->page_entry, 40, 0);
 	gtk_signal_connect (GTK_OBJECT (priv->page_entry), "activate", GTK_SIGNAL_FUNC (change_page_cmd), mp);
 	gtk_box_pack_start (GTK_BOX (status), priv->page_entry, FALSE, FALSE, 0);
+	gtk_label_set_mnemonic_widget ((GtkLabel *) l, priv->page_entry);
+
+	/* We are displaying 'Page: XXX of XXX'. */
+	gtk_box_pack_start (GTK_BOX (status), gtk_label_new (_("of")),
+			    FALSE, FALSE, 8);
+
 	priv->last = gtk_label_new ("");
 	gtk_box_pack_start (GTK_BOX (status), priv->last, FALSE, FALSE, 0);
+	atko = gtk_widget_get_accessible (priv->last);
+	atk_object_set_name (atko, _("Page total"));
+	atk_object_set_description (atko, _("The total number of pages in the document"));
 
 	gtk_box_pack_start (GTK_BOX (mp->vbox), status, FALSE, FALSE, 3);
 	gtk_box_pack_start (GTK_BOX (mp->vbox), priv->scrolled_window, TRUE, TRUE, 0);
 
+	/* Add a LABELLED_BY relation from the page entry to the label. */
+	atko = gtk_widget_get_accessible (priv->page_entry);
+	relation_set = atk_object_ref_relation_set (atko);
+	relation_targets[0] = gtk_widget_get_accessible (l);
+	relation = atk_relation_new (relation_targets, 1,
+				     ATK_RELATION_LABELLED_BY);
+	atk_relation_set_add (relation_set, relation);
+	g_object_unref (G_OBJECT (relation));
+	g_object_unref (G_OBJECT (relation_set));
+
 	gtk_widget_show_all (mp->vbox);
 
 	gtk_widget_grab_focus (GTK_WIDGET (priv->canvas));
@@ -673,37 +699,45 @@ create_toplevel (GnomePrintMasterPreview *mp)
 	width  = MIN (mp->paw + PAGE_PAD * 3, gdk_screen_width () - 40);
 	height = MIN (mp->pah + PAGE_PAD * 3, gdk_screen_height () - 40);
 
-	g_print ("width %d height %d\n", width, height);
-
 	gtk_window_set_policy (GTK_WINDOW (mp), TRUE, TRUE, FALSE);
 	gtk_widget_set_usize (GTK_WIDGET (mp), width, height);
 
 	/* This is not very beautiful, but works for now */
 
 	tb = gtk_toolbar_new ();
+	gtk_toolbar_set_style (GTK_TOOLBAR (tb), GTK_TOOLBAR_ICONS);
 	gtk_widget_show (tb);
 	gtk_box_pack_start (GTK_BOX (mp->vbox), tb, FALSE, FALSE, 0);
 
-	gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_PRINT, _("Prints the current file"), "", 
-				  GTK_SIGNAL_FUNC (preview_file_print_cmd), mp, 0);
-	gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_CLOSE, _("Closes print preview window"), "", 
-				  GTK_SIGNAL_FUNC (preview_close_cmd), mp, 1);
+	gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_PRINT,
+				  _("Prints the current file"), "", 
+				  GTK_SIGNAL_FUNC (preview_file_print_cmd), mp, -1);
+
+	gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_CLOSE,
+				  _("Closes print preview window"), "", 
+				  GTK_SIGNAL_FUNC (preview_close_cmd), mp, -1);
+
+	gtk_toolbar_append_space (GTK_TOOLBAR (tb));
+
 	mp->bpf = gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_GOTO_FIRST, _("Shows the first page"), "", 
-					    GTK_SIGNAL_FUNC (preview_first_page_cmd), mp, 2);
+					    GTK_SIGNAL_FUNC (preview_first_page_cmd), mp, -1);
 	mp->bpp = gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_GO_BACK, _("Shows the previous page"), "", 
-					    GTK_SIGNAL_FUNC (preview_prev_page_cmd), mp, 3);
+					    GTK_SIGNAL_FUNC (preview_prev_page_cmd), mp, -1);
 	mp->bpn = gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_GO_FORWARD, _("Shows the next page"), "", 
-					    GTK_SIGNAL_FUNC (preview_next_page_cmd), mp, 4);
+					    GTK_SIGNAL_FUNC (preview_next_page_cmd), mp, -1);
 	mp->bpl = gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_GOTO_LAST, _("Shows the last page"), "", 
-					    GTK_SIGNAL_FUNC (preview_last_page_cmd), mp, 5);
+					    GTK_SIGNAL_FUNC (preview_last_page_cmd), mp, -1);
+
+	gtk_toolbar_append_space (GTK_TOOLBAR (tb));
+
 	mp->bz1 = gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_ZOOM_100, _("Zooms 1:1"), "", 
-					    GTK_SIGNAL_FUNC (preview_zoom_100_cmd), mp, 6);
+					    GTK_SIGNAL_FUNC (preview_zoom_100_cmd), mp, -1);
 	mp->bzf = gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_ZOOM_FIT, _("Zooms to fit the whole page"), "", 
-					    GTK_SIGNAL_FUNC (preview_zoom_fit_cmd), mp, 7);
+					    GTK_SIGNAL_FUNC (preview_zoom_fit_cmd), mp, -1);
 	mp->bzi = gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_ZOOM_IN, _("Zooms the page in"), "", 
-					    GTK_SIGNAL_FUNC (gpmp_zoom_in_cmd), mp, 8);
+					    GTK_SIGNAL_FUNC (gpmp_zoom_in_cmd), mp, -1);
 	mp->bzo = gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_ZOOM_OUT, _("Zooms the page out"), "", 
-					    GTK_SIGNAL_FUNC (gpmp_zoom_out_cmd), mp, 9);
+					    GTK_SIGNAL_FUNC (gpmp_zoom_out_cmd), mp, -1);
 }
 
 /*
@@ -719,7 +753,7 @@ GtkType
 gnome_print_master_preview_get_type (void)
 {
 	static GtkType print_master_preview_type = 0;
-	
+
 	if (!print_master_preview_type) {
 		GtkTypeInfo print_master_preview_info = {
 			"GnomePrintMasterPreview",
@@ -729,13 +763,32 @@ gnome_print_master_preview_get_type (void)
 			(GtkObjectInitFunc) gnome_print_master_preview_init,
 			NULL, NULL, NULL
 		};
-		
+
 		print_master_preview_type = gtk_type_unique (GTK_TYPE_WINDOW, &print_master_preview_info);
 	}
-	
+
 	return print_master_preview_type;
 }
 
+static void
+gnome_print_master_preview_destroy (GtkObject *object)
+{
+	GnomePrintMasterPreview *pmp = GNOME_PRINT_MASTER_PREVIEW (object);
+	GPMPPrivate *priv = pmp->priv;
+
+	if (priv->preview != NULL) {
+		g_object_unref (G_OBJECT (priv->preview));
+		priv->preview = NULL;
+	}
+
+	if (priv->master != NULL) {
+		g_object_unref (G_OBJECT (priv->master));
+		priv->master = NULL;
+	}
+
+	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+}
+
 static void
 gnome_print_master_preview_finalize (GObject *object)
 {
@@ -745,16 +798,6 @@ gnome_print_master_preview_finalize (GObject *object)
 	pmp = GNOME_PRINT_MASTER_PREVIEW (object);
 	priv = pmp->priv;
 
-	if (priv && priv->preview != NULL) {
-		g_object_unref (G_OBJECT (priv->preview));
-		priv->preview = NULL;
-	}
-
-	if (priv && priv->master != NULL) {
-		g_object_unref (G_OBJECT (priv->master));
-		priv->master = NULL;
-	}
-
 	if (priv) {
 		g_free (priv);
 		pmp->priv = NULL;
@@ -766,13 +809,14 @@ gnome_print_master_preview_finalize (GObject *object)
 static void
 gnome_print_master_preview_class_init (GnomePrintMasterPreviewClass *klass)
 {
-	GObjectClass *object_class;
-	
-	object_class = (GObjectClass *) klass;
+	GtkObjectClass *object_class;
+
+	object_class = (GtkObjectClass *) klass;
 
 	parent_class = gtk_type_class (GTK_TYPE_WINDOW);
 
-	object_class->finalize = gnome_print_master_preview_finalize;
+	object_class->destroy = gnome_print_master_preview_destroy;
+	G_OBJECT_CLASS (object_class)->finalize = gnome_print_master_preview_finalize;
 }
 
 static void
@@ -828,7 +872,8 @@ gnome_print_master_preview_new (GnomePrintMaster *gpm, const guchar *title)
 
 	goto_page (gpmp, 0);
 	
-	text = g_strdup_printf ("/%d", priv->pagecount);
+	text = g_strdup_printf ("%d", priv->pagecount);
+
 	gtk_label_set_text (GTK_LABEL (priv->last), text);
 	g_free (text);
 
@@ -840,10 +885,10 @@ gnome_print_master_preview_new (GnomePrintMaster *gpm, const guchar *title)
  * gnome_print_master_preview_new:
  * @gpm: A GnomePrintMaster which has been printed to and completed.
  * @title: Window title for the preview window.
- * 
+ *
  * Create a new preview window widget.  The preview has inbuilt
  * buttons for zooming and panning, and menu's to print.
- * 
+ *
  * Return value: A newly created GnomePrintMasterPreview widget.
  **/
 GnomePrintMasterPreview *
@@ -857,10 +902,10 @@ gnome_print_master_preview_new (GnomePrintMaster *gpm, const guchar *title)
  * @gpm: A GnomePrintMaster which has been printed to and completed.
  * @title: Window title for the preview window.
  * @landscape: TRUE to rotate paper.
- * 
+ *
  * Create a new preview window widget.  The preview has inbuilt
  * buttons for zooming and panning, and menu's to print.
- * 
+ *
  * Return value: A newly created GnomePrintMasterPreview widget.
  **/
 GnomePrintMasterPreview *
@@ -887,12 +932,12 @@ gnome_print_master_preview_new_with_orientation (GnomePrintMaster *gpm,
 
 	/* this zooms to fit, once we know how big the window actually is */
 	gtk_signal_connect (GTK_OBJECT (pp->canvas), "realize",
-			   GTK_SIGNAL_FUNC (preview_zoom_fit_cmd), pmp);
+			    GTK_SIGNAL_FUNC (preview_zoom_fit_cmd), pmp);
 
 	pp->pagecount = gnome_print_master_get_pages (gpm);
 
 	goto_page (pmp, 0);
-	
+
 	g_snprintf (text, 4 * sizeof (int), "/%d", pp->pagecount);
 	gtk_label_set_text (GTK_LABEL (pp->last), text);
 
