@@ -244,10 +244,14 @@ test_ps (GnomePrintConfig *config, GnomeFont *font, gboolean preview)
 			GnomePrintContext *ctx;
 			GtkWidget *gpmp;
 			if (config) {
+#if 0	
 				g_print ("We have config\n");
+#endif	
 				gpm = gnome_print_master_new_from_config (config);
 			} else {
+#if 0	
 				g_print ("Do not have config\n");
+#endif	
 				gpm = gnome_print_master_new ();
 			}
 			ctx = gnome_print_master_get_context (gpm);
